@@ -11,7 +11,7 @@ from josepy.b64 import b64decode
 from josepy.jwk import JWK
 from josepy.jws import JWS, Header
 from pretix.base.auth import BaseAuthBackend
-from pretix.base.models import User
+from pretix.base.models import User, Organizer, Team, Event
 from pretix.base.models.auth import EmailAddressTakenError
 from pretix.settings import config
 from requests.auth import HTTPBasicAuth
@@ -266,12 +266,41 @@ class OIDCAuthBackend(BaseAuthBackend):
         # email based filtering
         # users = self.filter_users_by_claims(user_info)
 
+        try:
+            email = user_info["email"]
+        except KeyError:
+            messages.error(self.request,
+                           _("Error: Email missing from keycloak roles"), )
+            return redirect(reverse("control:auth.login"))
+
+        try:
+            pk = user_info["sub"]
+        except KeyError:
+            messages.error(self.request,
+                           _("Error: sub missing from keycloak roles"), )
+            return redirect(reverse("control:auth.login"))
+
+        try:
+            username = user_info["preferred_username"]
+        except KeyError:
+            username = user_info["email"]
+
+        new_roles = set(user_info['roles']) if user_info['roles'] else set()
+        if 'si-gestor-eventos-admin' in new_roles:
+            is_staff = True
+            new_roles.remove('si-gestor-eventos-admin')
+        else:
+            is_staff = False
+
         try:
             u = User.objects.get_or_create_for_backend(
                 "keycloak_auth",
-                user_info["sub"],
-                user_info["email"],
-                set_always={},
+                pk,
+                email,
+                set_always={
+                    'is_staff': is_staff,
+                    'fullname': username,
+                },
                 set_on_creation={},
             )
         except EmailAddressTakenError:
@@ -283,6 +312,112 @@ class OIDCAuthBackend(BaseAuthBackend):
                 ),
             )
             return redirect(reverse("control:auth.login"))
+
+        # Update Organizer Units from Roles
+        prev_roles = set(Organizer.objects.filter(
+            id__in=u.teams.filter(name="Administrators").values_list('organizer', flat=True)
+        ).values_list('slug', flat=True))
+
+        if prev_roles != new_roles:
+            for role in set(new_roles):
+                # Add new roles
+                obj, created = Organizer.objects.get_or_create(slug=role, name=role)
+
+                if not created:
+                    # Check if has Manager Unit team
+                    try:
+                        t = Team.objects.get(organizer=obj, name='Managing Units')
+                        continue # Team is setted correctly
+                    except Team.DoesNotExist:
+                        pass
+                    except Team.MultipleObjectsReturned:
+                        pass
+                    # Fix teams
+                    for team in obj.teams.all():
+                        team.delete()
+
+                # Create main team that materializes a Managing Unit from Event Manager
+                t = Team.objects.create(
+                    organizer=obj, name='Managing Units',
+                    all_events=True, can_create_events=True, can_change_teams=True,
+                    can_manage_gift_cards=True,
+                    can_change_organizer_settings=True,
+                    can_change_event_settings=True,
+                    can_change_items=True,
+                    can_manage_customers=True,
+                    can_view_orders=True, can_change_orders=True,
+                    can_view_vouchers=True, can_change_vouchers=True
+                )
+                t.members.add(u)
+
+            for role in prev_roles - new_roles:
+                # Remove user from removed roles
+                obj = Organizer.objects.get(slug=role)
+
+                # Check if has Manager Unit team
+                try:
+                    t = Team.objects.get(organizer=obj, name='Managing Units')
+                    t.members.remove(u)
+                    continue  # Team is removed correctly
+                except Team.DoesNotExist:
+                    pass
+                except Team.MultipleObjectsReturned:
+                    pass
+                # Fix teams
+                for team in obj.teams.all():
+                    team.delete()
+
+        # Update non Mananging Unit Event Manager
+        url = self.get_settings("UP_EVENT_MANAGER_API", "https://eventos.dev.uporto.pt/backoffice/api/user")
+        url = f"{url}/{u.auth_backend_identifier}/"
+
+        headers = {
+            'Content-Type': 'application/json',
+            'Accept': 'application/json, text/javascript',
+            'Authorization': f"Token {self.get_settings('UP_EVENT_MANAGER_TOKEN', None)}"
+        }
+        r = requests.get(url, headers=headers)
+        if r.status_code != 200:
+            messages.error(self.request,
+                           _("Error Gestor Eventos API"), )
+            return redirect(reverse("control:auth.login"))
+
+        events = r.json()
+
+        # Remove user from all other non Mananging Unit Event Manager
+        # (By deleting them since they are single user Teams)
+        # u.teams.exclude(name='Managing Units').delete()
+        #
+        # for slug in events:
+        #     # Get Event
+        #     try:
+        #         event = Event.objects.get(slug=slug)
+        #     except Event.DoesNotExist:
+        #         msg = f"Event {slug} from Gestor de Eventos UP does not exist on Pretix"
+        #         raise SuspiciousOperation(msg)
+        #
+        #     try:
+        #         t = Team.objects.get(organizer=obj, name=f'Manager of {slug}')
+        #         t.members.add(u)
+        #         continue
+        #     except Team.MultipleObjectsReturned:
+        #         pass
+        #     except Team.DoesNotExist:
+        #         pass
+        #     t = Team.objects.get_or_create(
+        #         organizer=obj, name=f'Manager of {slug}',
+        #         all_events=False, can_create_events=False, can_change_teams=False,
+        #         can_manage_gift_cards=True,
+        #         can_change_organizer_settings=False,
+        #         can_change_event_settings=True,
+        #         can_change_items=True,
+        #         can_manage_customers=True,
+        #         can_view_orders=True, can_change_orders=True,
+        #         can_view_vouchers=True, can_change_vouchers=True
+        #     )
+        #     t.limit_events.add(event)
+        #     t.members.add(u)
+
         return u
 
     def get_userinfo(self, access_token, id_token, payload):
